import subprocess
import sys

def auto_installer():
    packages = list(set([
        "cosmodb",
        "sheetsmart"
    ]))

    for package_name in packages:
        try:
            # Check if the package is already installed
            check = subprocess.run(
                [sys.executable, "-m", "pip", "show", package_name],
                capture_output=True, text=True
            )

            if check.returncode == 0:
                continue  # Already installed, skip all output

            # If not installed, install and show minimal output
            print(f"Please wait, installing {package_name}...")

            install = subprocess.run(
                [sys.executable, "-m", "pip", "install", package_name],
                capture_output=True, text=True
            )

            if install.returncode != 0:
                continue  # Fail silently as per instructions

            # Fetch installed version
            version = "Unknown"
            version_check = subprocess.run(
                [sys.executable, "-m", "pip", "show", package_name],
                capture_output=True, text=True
            )
            if version_check.returncode == 0:
                for line in version_check.stdout.splitlines():
                    if line.startswith("Version:"):
                        version = line.split(":")[1].strip()
                        break

            print(f"Installed version: {version}")

        except Exception:
            pass  # Fully silent on error





import difflib
import subprocess
import sys
import importlib.util
import os

def chat():
    def is_relevant(line, topic):
        keywords_map = {
            "Sun": ["plasma", "hydrogen", "helium", "nuclear fusion", "solar system"],
            "Mars": ["red planet", "perseverance", "thin atmosphere", "water", "habitable"],
            "Europa": ["jupiter", "ice", "ocean", "life", "cracks"],
            "Jupiter": ["gas giant", "ganymede", "storm", "red spot"],
            "Venus": ["cloud", "greenhouse", "volcano", "carbon dioxide"],
            "Earth": ["life", "ecosystem", "magnetic", "oxygen"],
            "Mercury": ["temperature", "closest", "orbit", "smallest"],
            "Saturn": ["rings", "hydrogen", "titan", "storm"],
            "Uranus": ["ice giant", "tilt", "methane", "cold"],
            "Neptune": ["winds", "storm", "blue", "triton"],
            "Pluto": ["dwarf", "kuiper", "charon", "glacier"],
            "Ceres": ["asteroid belt", "briny", "dwarf", "ice"],
            "DESTINY+": ["phaethon", "dust", "deep-space", "geminid"],
            "SLIM": ["lunar", "landing", "moon", "jaxa"],
            "MMX": ["mars", "phobos", "deimos"],
            "HTV": ["iss", "cargo", "resupply"],
            "IKAROS": ["solar sail", "radiation", "interplanetary"],
            "Akatsuki": ["venus", "atmosphere", "super-rotation"],
            "Ariane": ["launcher", "satellites", "esa", "ariane 5"],
            "EarthCARE": ["cloud", "aerosol", "earth radiation"],
            "EnVision": ["venus", "radar", "evolution"],
            "Hera": ["dart", "dimorphos", "asteroid"],
        }

        keywords = keywords_map.get(topic, [topic.lower()])
        return any(keyword in line.lower() for keyword in keywords)

    def identify_and_store(file_path, output_path):
        with open(file_path, "r", encoding="utf-8") as file:
            lines = file.readlines()

        with open(output_path, "w", encoding="utf-8") as out:
            out.write("🌌 Celestial Info Bot Knowledge Base:\n\n")
            for line in lines:
                parts = line.strip().split(" ", 1)
                if len(parts) > 1:
                    topic = parts[0]
                    if is_relevant(line, topic):
                        out.write(f"== Topic: {topic} ==\n")
                        out.write(line.strip() + "\n\n")
                    else:
                        out.write(f"-- Mismatched Topic: {topic} (Skipped)\n\n")

    def is_installed(package_name):
        return importlib.util.find_spec(package_name) is not None

    if not is_installed("sheetsmart"):
        try:
            subprocess.check_call([sys.executable, "-m", "pip", "install", "sheetsmart"])
        except subprocess.CalledProcessError:
            print("❌ Failed to install SheetSmart")

    def load_knowledge(file_path):
        with open(file_path, "r", encoding="utf-8") as file:
            content = file.read()

        topics = {}
        current_topic = None
        for line in content.splitlines():
            if line.startswith("== Topic: "):
                current_topic = line.strip().replace("== Topic: ", "").replace(" ==", "")
                topics[current_topic.lower()] = ""
            elif current_topic:
                topics[current_topic.lower()] += line.strip() + " "
        return topics

    def find_relevant_topics(user_input, topics, threshold=0.6):
        matches = set()
        words = user_input.lower().replace(",", " ").split()
        for word in words:
            for topic in topics:
                similarity = difflib.SequenceMatcher(None, word, topic).ratio()
                if similarity >= threshold:
                    matches.add(topic)
        return list(matches)

    def generate_response(user_input, topics):
        matched_topics = find_relevant_topics(user_input, topics)
        if matched_topics:
            response = ""
            for topic in matched_topics:
                response += f"\n🌠 {topic.title()}:\n{topics[topic].strip()}\n"
            return response
        else:
            return "CosmoTalker: I'm sorry, I couldn't find any information on that. Try asking about a specific planet, satellite, or celestial object."

    # ✅ Locate project root and make data directory
    current_dir = os.path.dirname(os.path.dirname(__file__))  # one level up from cosmotalker/
    data_dir = os.path.join(current_dir, "data")
    os.makedirs(data_dir, exist_ok=True)
    output_file_path = os.path.join(data_dir, "celestial_knowledge.txt")

    if not os.path.exists("ss.txt"):
        print("❌ 'ss.txt' not found in the current directory.")
        return

    identify_and_store("ss.txt", output_file_path)
    knowledge = load_knowledge(output_file_path)

    print("🌌 CosmoTalker Chatbot (type 'exit' to quit)")
    query_count = 0

    while True:
        user_input = input("You: ")
        if user_input.lower() == "exit":
            break

        response = generate_response(user_input, knowledge)
        print(response + "\n")

        query_count += 1
        if query_count % 5 == 0:
            print("📣 Try the SheetSmart library! Developed by Bhuvanesh M — used widely for data manipulation in Python.\n✨ Just run: import sheetsmart\n")



import subprocess
import sys

def update():
    packages = [
        "cosmodb",
        "customtkinter",
        "pillow",
        "requests",
        "pyperclip",
        "cosmotalker",
        "sheetsmart",
        "cosmotalker"
    ]

    for package_name in packages:
        try:
            # Get current version
            old = subprocess.run(
                [sys.executable, "-m", "pip", "show", package_name],
                capture_output=True, text=True
            )
            old_version = "None"
            if old.returncode == 0:
                for line in old.stdout.splitlines():
                    if line.startswith("Version:"):
                        old_version = line.split(":")[1].strip()
                        break
            print(f"\n🔍 Current version of '{package_name}': {old_version}")

            # Uninstall
            print(f"🧹 Uninstalling '{package_name}'...")
            subprocess.run(
                [sys.executable, "-m", "pip", "uninstall", "-y", package_name],
                capture_output=True, text=True
            )

            # Reinstall latest version
            print(f"📦 Reinstalling latest version of '{package_name}'...")
            install = subprocess.run(
                [sys.executable, "-m", "pip", "install", package_name],
                capture_output=True, text=True
            )

            # Get new version
            new = subprocess.run(
                [sys.executable, "-m", "pip", "show", package_name],
                capture_output=True, text=True
            )
            new_version = "Unknown"
            if new.returncode == 0:
                for line in new.stdout.splitlines():
                    if line.startswith("Version:"):
                        new_version = line.split(":")[1].strip()
                        break

            # Final message
            if install.returncode == 0:
                print(f"\n✅ '{package_name}' updated: {old_version} → {new_version}")
            else:
                print(f"\n❌ Installation failed for '{package_name}':\n{install.stderr}")

            print("\n" + "="*15 + f" Finished: {package_name} " + "="*15 + "\n")

        except Exception as e:
            print(f"\n❗ Error with '{package_name}': {e}")




def img():
    import cosmodb
    import tkinter as tk
    import customtkinter as ctk
    from PIL import Image, ImageTk
    import requests
    from io import BytesIO
    import threading
    import webbrowser
    from tkinter import filedialog
    import pyperclip



    # Theme Setup
    ctk.set_appearance_mode("Dark")
    ctk.set_default_color_theme("dark-blue")

    class ImageApp(ctk.CTk):
        def __init__(self):
            super().__init__()
            self.title("Image Viewer by CosmoTalker")
            self.geometry("650x600")
            self.configure(bg="#1e1e2f")
            self.resizable(True, True)

            self.fullscreen = True
            self.current_image_url = None
            self.current_image_data = None
            self.timer_id = None

            # Fade in/out on open/close
            self.attributes('-alpha', 0.0)
            self.after(0, self.fade_in_window)
            self.protocol("WM_DELETE_WINDOW", self.fade_out_window)

            self.bind("<F11>", self.toggle_fullscreen)
            self.bind("<Escape>", self.exit_fullscreen)

            # UI Widgets
            self.label = ctk.CTkLabel(self, text="Enter Image Name:",
                                      font=("Arial", 20, "bold"), text_color="white")
            self.label.pack(pady=20)

            self.entry = ctk.CTkEntry(self, placeholder_text="Type: Earth, Sun , Neptune ...",
                                      font=("Arial", 16), width=300)
            self.entry.pack(pady=10)
            self.entry.bind('<Return>', lambda event: self.check_input())

            self.button = ctk.CTkButton(self, text="Show Image", width=300,
                                        fg_color="#a020f0", hover_color="#c070f0",
                                        text_color="white", font=("Arial", 16),
                                        command=self.check_input)
            self.button.pack(pady=10)

            self.image_label = tk.Label(self, bg="#1e1e2f")
            self.image_label.pack(pady=20)

            self.url_button = ctk.CTkButton(self, text="Copy Image URL", width=200,
                                            fg_color="#5a4fbf", hover_color="#8675f0",
                                            text_color="white", font=("Arial", 14),
                                            command=self.copy_url, state="disabled")
            self.url_button.pack(pady=5)

            self.download_button = ctk.CTkButton(self, text="Download Image", width=200,
                                                 fg_color="#5a4fbf", hover_color="#8675f0",
                                                 text_color="white", font=("Arial", 14),
                                                 command=self.download_image, state="disabled")
            self.download_button.pack(pady=5)

            self.verify_button = ctk.CTkButton(self, text="Verify Website", width=200,
                                               fg_color="#3b7ddd", hover_color="#5a9eff",
                                               text_color="white", font=("Arial", 14),
                                               command=self.verify_website)
            self.verify_button.pack(pady=5)

            self.toast_label = tk.Label(self, bg="#1e1e2f", fg="lightgreen", font=("Arial", 12))
            self.toast_label.pack_forget()

            # Disclaimer
            disclaimer_text = ('''Disclaimer: The images displayed in this project are fetched from publicly available sources on the internet through search engines such as Google, Bing, DuckDuckGo, Ecosia, and others. These images are not created, owned, or developed by CosmoTalker or its developer. All rights to the images remain with their original creators or copyright holders.\n\n

CosmoTalker is a free and open-source Python library. This project is not monetized in any form. Every image used through the img() function in CosmoTalker is selected only after confirming that it is licensed under a Creative Commons (CC) or similar open license. The links provided through the "Verify Website" button lead directly to the original source of the image, where license information and proper credit can be viewed.\n\n

Although the images are initially used under Creative Commons terms, it is possible that license terms may change over time. In such rare cases, if you are the copyright holder and believe that an image is being used improperly, you may notify the creator of this project (BHUVANESH M), CosmoTalker. Upon appropriate notice and verification, the image will be promptly reviewed and removed.\n\n

Thank you for your understanding and support. CosmoTalker is committed to respecting the rights of all creators, maintaining transparency, and ensuring responsible use of digital content.'''
            )
            self.disclaimer_label = ctk.CTkLabel(self, text=disclaimer_text,
                                                 font=("Arial", 10), text_color="gray", wraplength=600)
            self.disclaimer_label.pack(pady=10)

        # Fade animation
        def fade_in_window(self, alpha=0.0):
            if alpha < 1.0:
                self.attributes('-alpha', alpha)
                self.after(30, lambda: self.fade_in_window(alpha + 0.05))
            else:
                self.attributes('-alpha', 1.0)

        def fade_out_window(self, alpha=1.0):
            if alpha > 0:
                self.attributes('-alpha', alpha)
                self.after(30, lambda: self.fade_out_window(alpha - 0.05))
            else:
                self.destroy()

        def toggle_fullscreen(self, _=None):
            self.fullscreen = not self.fullscreen
            self.attributes('-fullscreen', self.fullscreen)

        def exit_fullscreen(self, _=None):
            self.fullscreen = False
            self.attributes('-fullscreen', False)

        # Fetch & Display
        def check_input(self):
            keyword = self.entry.get().strip().lower()
            url = cosmodb.image(keyword)
            if url:
                self.current_image_url = url
                self.image_label.config(image='', text="Loading...")
                threading.Thread(target=self.load_image, args=(url,)).start()
            else:
                self.image_label.config(image='', text="❌ No image found!", fg="white")
                self.url_button.configure(state="disabled")
                self.download_button.configure(state="disabled")

        def load_image(self, url):
            try:
                response = requests.get(url)
                image = Image.open(BytesIO(response.content)).convert("RGBA")
                image = image.resize((300, 300), Image.LANCZOS)
                self.current_image_data = image
                self.fade_in_image(image, 0.0)
                self.url_button.configure(state="normal")
                self.download_button.configure(state="normal")
            except Exception:
                self.image_label.config(text="⚠️ Error loading image!", fg="white")

        def fade_in_image(self, pil_image, alpha):
            if alpha >= 1.0:
                photo = ImageTk.PhotoImage(pil_image)
                self.image_label.config(image=photo, text='')
                self.image_label.image = photo
            else:
                faded = pil_image.copy()
                faded.putalpha(int(255 * alpha))
                faded = faded.convert("RGBA")
                photo = ImageTk.PhotoImage(faded)
                self.image_label.config(image=photo, text='')
                self.image_label.image = photo
                self.after(50, lambda: self.fade_in_image(pil_image, alpha + 0.1))

        def copy_url(self):
            if self.current_image_url:
                pyperclip.copy(self.current_image_url)
                self.animate_button_text(self.url_button, "✅ URL Copied!", "Copy Image URL", 3000)

        def animate_button_text(self, button, temp_text, original_text, duration_ms):
            button.configure(text=temp_text)
            self.after(duration_ms, lambda: button.configure(text=original_text, text_color="white"))

        def download_image(self):
            if self.current_image_data:
                keyword = self.entry.get().strip().lower()
                file_path = filedialog.asksaveasfilename(
                    defaultextension=".png",
                    filetypes=[("PNG files", "*.png")],
                    initialfile=f"{keyword}_cosmotalker.png",
                    title="Save Image As"
                )
                if file_path:
                    try:
                        self.current_image_data.save(file_path, format="PNG")
                        self.show_toast("✅ Image downloaded successfully!")
                    except Exception:
                        self.show_toast("❌ Failed to save image!")

        def verify_website(self):
            keyword = self.entry.get().strip().lower()
            source_url = cosmodb.image_source(keyword)
            if source_url:
                webbrowser.open(source_url)
                self.show_toast("🌐 Opened source website!")
            else:
                self.show_toast("❌ Source website not found!")

        # Toast Notifications
        def show_toast(self, message):
            if self.timer_id:
                self.after_cancel(self.timer_id)

            self.toast_label.config(text=f"{message} (5s)", fg="lightgreen")
            self.toast_label.pack(pady=10)
            self.timer_counter = 5
            self.update_toast_timer()

            if not hasattr(self, 'close_toast_button'):
                self.close_toast_button = ctk.CTkButton(self, text="Dismiss", width=100,
                                                        fg_color="#ff5555", hover_color="#ff7777",
                                                        command=self.hide_toast)
                self.close_toast_button.pack(pady=5)

        def update_toast_timer(self):
            if self.timer_counter <= 0:
                self.hide_toast()
            else:
                self.toast_label.config(text=self.toast_label.cget("text").split('(')[0] + f" ({self.timer_counter}s)")
                self.timer_counter -= 1
                self.timer_id = self.after(1000, self.update_toast_timer)

        def hide_toast(self):
            self.toast_label.pack_forget()
            if hasattr(self, 'close_toast_button'):
                self.close_toast_button.pack_forget()

    app = ImageApp()
    app.mainloop()
img()



from .auto_installer import auto_installer
auto_installer()
from .img import img
from .get import get
from .spacex import spacex
from .search import search
from .apod import apod
from .celestrak import celestrak
from .feedback import feedback
from .chat import chat
from .wiki import wiki
from .update import update

